<?php
/* Copyright (C) 2015		Charles-fr BENKE 			<charles.fr@benke.fr>
 * Copyright (C) 2016		Philippe SAGOT (Philazerty)	<courrier@mon-dolibarr.fr>
 * Copyright (C) 2017	    Laurent DINCLAUX (loxK)     <laurent@gecka.nc>
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 * or see http://www.gnu.org/
 */

/**
 *        \file        geckanewcaledonia/core/lib/main.lib.php
 *        \brief        Functions used by geckanewcaledonia module
 *        \ingroup    geckanewcaledonia
 */

function geckanewcaledonia_admin_prepare_head()
{
    global $langs, $conf, $user;

    $h    = 0;
    $head = array();

    $head[ $h ][0] = dol_buildpath('/geckanewcaledonia/admin/setup.php', 1);
    $head[ $h ][1] = $langs->trans("Setup");
    $head[ $h ][2] = 'setup';

    $h ++;
    $head[ $h ][0] = dol_buildpath('/geckanewcaledonia/admin/setup-dictionaries.php', 1);
    $head[ $h ][1] = $langs->trans("DictionarySetup");
    $head[ $h ][2] = 'setup-dictionaries';

    $h ++;
    $head[ $h ][0] = dol_buildpath('/geckanewcaledonia/admin/guide.php', 1);
    $head[ $h ][1] = $langs->trans("GeckaNCGuide");
    $head[ $h ][2] = 'guide';

    $h ++;
    $head[ $h ][0] = dol_buildpath('/geckanewcaledonia/admin/changelog.php', 1);
    $head[ $h ][1] = $langs->trans("ChangeLog");
    $head[ $h ][2] = 'changelog';

    $h ++;
    $head[ $h ][0] = dol_buildpath('/geckanewcaledonia/admin/about.php', 1);
    $head[ $h ][1] = $langs->trans("About");
    $head[ $h ][2] = 'about';

    return $head;
}

function geckanewcaledonia_admin_nc_country_id()
{
    global $db;

    $sql   = "SELECT rowid  FROM `" . MAIN_DB_PREFIX . "c_country` WHERE `code` LIKE 'NC'";
    $resql = $db->query($sql);

    dol_syslog(__FUNCTION__ . " " . $sql);
    if ($resql) {
        return $resql->fetch_object()->rowid;
    }

    return false;
}

function geckanewcaledonia_admin_dump_tax_data($replace = false)
{
    global $db;

    $replace = $replace ? true : false;

    $ncID = geckanewcaledonia_admin_nc_country_id();
    if (! $ncID) {
        return 0;
    }

    if ($replace) {
        $sql   = "DELETE FROM `" . MAIN_DB_PREFIX . "c_tva` WHERE `fk_pays` = " . $ncID;
        $resql = $db->query($sql);

        if (! $resql) {
            return 0;
        }
    }

    $sql = "INSERT INTO `" . MAIN_DB_PREFIX . "c_tva` (`fk_pays`, `code`, `taux`, `localtax1`, `localtax1_type`, `localtax2`, `localtax2_type`, `recuperableonly`, `note`, `active`, `accountancy_code_sell`, `accountancy_code_buy`) VALUES
			($ncID, 'HT', 0, '0', '0', '0', '0', 0, 'Taux 0 ou non applicable', 1, NULL, NULL),
			($ncID, 'HT-TSS', 0, '5', '1', '0', '0', 0, 'Taux 0 ou non applicable + TSS 5%', 1, NULL, NULL),
			($ncID, 'TGC025', 0.25, '0', '0', '0', '0', 0, 'Taux réduit (0.25%)', 1, NULL, NULL),
			($ncID, 'TGC025-TSS', 0.25, '5', '1', '0', '0', 0, 'Taux réduit (0.25%) + TSS 5%', 1, NULL, NULL),
			($ncID, 'TGC035', 0.35, '0', '0', '0', '0', 0, 'Taux spécifique (0.35%)', 1, NULL, NULL),
			($ncID, 'TGC035-TSS', 0.35, '5', '1', '0', '0', 0, 'Taux spécifique (0.35%) + TSS 5%', 1, NULL, NULL),
			($ncID, 'TGC05', 0.5, '0', '0', '0', '0', 0, 'Taux normal (0.5%)', 1, NULL, NULL),
			($ncID, 'TGC05-TSS', 0.5, '5', '1', '0', '0', 0, 'Taux normal (0.5%) + TSS 5%', 1, NULL, NULL),
			($ncID, 'TGC1', 1, '0', '0', '0', '0', 0, 'Taux supérieur (1%)', 1, NULL, NULL),
			($ncID, 'TGC1-TSS', 1, '5', '1', '0', '0', 0, 'Taux supérieur (1%) + TSS 5%', 1, NULL, NULL);";

    $resql = $db->query($sql);
    if ($resql && mysqli_affected_rows($db->db)) {
        return 1;
    }

    return 0;
}

function geckanewcaledonia_admin_dump_legal_form($replace = false)
{
    global $db;

    $replace = $replace ? true : false;

    $ncID = geckanewcaledonia_admin_nc_country_id();
    if (! $ncID) {
        return 0;
    }

    if ($replace) {
        $sql   = "DELETE FROM `" . MAIN_DB_PREFIX . "c_forme_juridique` WHERE `fk_pays` = " . $ncID;
        $resql = $db->query($sql);

        if (! $resql) {
            return 0;
        }
    }

    $sql   = "SELECT * FROM `" . MAIN_DB_PREFIX . "c_forme_juridique` WHERE `fk_pays` = 1";
    $resql = $db->query($sql);

    $sql  = "INSERT INTO `" . MAIN_DB_PREFIX . "c_forme_juridique` (`code`, `fk_pays`, `libelle`, `isvatexempted`, `active`, `module`, `position`) VALUES\n";
    $rows = [];
    while ($row = mysqli_fetch_assoc($resql)) {
        $code    = (int) ( $ncID . $row['code'] );
        $libelle = addslashes($row['libelle']);
        $rows[]  = "($code, {$ncID}, '{$libelle}', {$row['isvatexempted']}, {$row['active']}, '{$row['module']}', {$row['position']})";
    }
    $sql .= implode(",\n", $rows);
    $sql .= ";";

    $resql = $db->query($sql);
    if ($resql && mysqli_affected_rows($db->db)) {
        return 1;
    }

    return 0;
}

function geckanewcaledonia_admin_dump_langs($replace = false)
{
    global $db;

    $replace = $replace ? true : false;

    if ($replace) {
        $sql   = "DELETE FROM `" . MAIN_DB_PREFIX . "overwrite_trans` WHERE `lang` = 'fr_NC'";
        $resql = $db->query($sql);

        if (! $resql) {
            return 0;
        }
    }

    $sql = "INSERT INTO `" . MAIN_DB_PREFIX . "overwrite_trans` (`entity`, `lang`, `transkey`, `transvalue`) VALUES 
			(1, 'fr_NC', 'AmountVAT', 'Montant TGC'),
			(1, 'fr_NC', 'MulticurrencyAmountVAT', 'Montant TGC, devise d\'origine'),
			(1, 'fr_NC', 'TotalVAT', 'Total TGC'),
			(1, 'fr_NC', 'VAT', 'TGC'),
			(1, 'fr_NC', 'VATs', 'TGC'),
			(1, 'fr_NC', 'VATRate', 'Taux TGC'),
			(1, 'fr_NC', 'NoVAT', 'Pas de TGC pour cette vente'),
			(1, 'fr_NC', 'ErrorVATIntraNotConfigured', 'Numéro de TVA intracommunautaire pas encore configuré'),
			(1, 'fr_NC', 'ConfirmClassifyPaidPartiallyReasonDiscountNoVat', 'Le reste à payer <b>(%s %s)</b> est un escompte accordé après facture. J\'accepte de perdre la TGC sur cet escompte'),
			(1, 'fr_NC', 'IntracommunityVATNumber', 'Numéro de TVA intracommunautaire'),
			(1, 'fr_NC', 'HideAnyVATInformationOnPDF', 'Cacher toutes les informations en rapport avec la TGC sur les PDF générés'),
			(1, 'fr_NC', 'OldVATRates', 'Ancien taux de TGC'),
			(1, 'fr_NC', 'NewVATRates', 'Nouveau taux de TGC'),
			(1, 'fr_NC', 'ShowDetailsInPDFPageFoot', 'Ajout de plus d\'informations dans le pied de page de vos documents pdf, comme l\'adresse de votre société, ou le nom du directeur (pour compléter les identifiants professionnels, le capital de la société et le numéro de TGC intra communautaire)'),
			(1, 'fr_NC', 'Permission91', 'Lire les charges fiscales ou sociales et paiement de TGC'),
			(1, 'fr_NC', 'Permission92', 'Créer/modifier les charges fiscales ou sociales et paiements de TGC'),
			(1, 'fr_NC', 'Permission93', 'Effacer les charges fiscales ou sociales et paiements de TGC'),
			(1, 'fr_NC', 'DictionaryVAT', 'Taux de TGC ou de Taxes de Ventes'),
			(1, 'fr_NC', 'VATManagement', 'Gestion TGC'),
			(1, 'fr_NC', 'VATIsUsedDesc', 'Le taux de TGC proposé par défaut lors de la création de proposition commerciale, facture, commande, etc... répond à la règle standard suivante :<br>Si vendeur non assujetti à TGC, TGC par défaut=0. Fin de règle.<br>Si le (pays vendeur= pays acheteur) alors TGC par défaut=TGC du produit vendu. Fin de règle.<br>Si vendeur et acheteur dans Communauté européenne et bien vendu= moyen de transport neuf (auto, bateau, avion), TGC par défaut=0 (La TGC doit être payée par acheteur au centre d\'impôts de son pays et non au vendeur). Fin de règle.<br>Si vendeur et acheteur dans Communauté européenne et acheteur= particulier alors TGC par défaut=TGC du produit vendu (TGC pays vendeur si < seuil du pays et si  avant 01/01/2015, TGC pays acheteur après le 01/01/2015). Fin de règle.<br>Si vendeur et acheteur dans Communauté européenne et acheteur= entreprise alors TGC par défaut=0. Fin de règle.<br>Sinon TGC proposée par défaut=0. Fin de règle.<br>'),
			(1, 'fr_NC', 'VATIsNotUsedDesc', 'Le taux de TGC proposé par défaut est 0. C\'est le cas d\'associations, particuliers ou certaines petites sociétés.'),
			(1, 'fr_NC', 'VATIsUsedExampleFR', 'En France, il s\'agit des sociétés ou organismes ayant choisi un régime fiscale réel (Réel simplifié ou Réel normal), régime dans lequel la TGC est déclarée.'),
			(1, 'fr_NC', 'VATIsNotUsedExampleFR', 'En France, il s\'agit des associations ne déclarant pas de TGC ou sociétés, organismes ou professions libérales ayant choisi le régime fiscal micro entreprise (TGC en franchise) et payant une TGC en franchise sans faire de déclaration de TGC. Ce choix fait de plus apparaître la mention \"TGC non applicable - art - 293B du CGI\" sur les factures.'),
			(1, 'fr_NC', 'LocalTax1IsUsedDesc', 'Utilisation d\'un 2ème type taxe (autre que TGC)'),
			(1, 'fr_NC', 'LocalTax1IsNotUsedDesc', 'Pas d\'utilisation de 2ème type taxe (autre que TGC)'),
			(1, 'fr_NC', 'LocalTax2IsUsedDesc', 'Utilisation d\'un 3ème type taxe (autre que TGC)'),
			(1, 'fr_NC', 'LocalTax2IsNotUsedDesc', 'Pas d\'utilisation de 3ème type taxe (autre que TGC)'),
			(1, 'fr_NC', 'TotalPriceAfterRounding', 'Prix total (HT/TGC/TTC) après arrondis'),
			(1, 'fr_NC', 'ShowVATIntaInAddress', 'Cacher l\'identifiant de TGC Intracommunautaire dans les adresses sur les documents'),
			(1, 'fr_NC', 'TaxSetup', 'Configuration du module TGC, charges fiscales ou sociales et dividendes'),
			(1, 'fr_NC', 'OptionVatMode', 'Option d\'exigibilité de TGC par défaut'),
			(1, 'fr_NC', 'OptionVatDefaultDesc', 'TGC sur encaissement, l\'exigibilité de la TGC est:<br>- sur livraison pour les biens (en pratique on utilise la date de facturation)<br>- sur paiement pour les services'),
			(1, 'fr_NC', 'OptionVatDebitOptionDesc', 'TGC sur débit, l\'exigibilité de la TGC est:<br>- sur livraison pour les biens (en pratique on utilise la date de facturation)<br>- sur facturation (débit) pour les services'),
			(1, 'fr_NC', 'SummaryOfVatExigibilityUsedByDefault', 'Moment d\'exigibilité par défaut de la TGC pour l\'option choisie:'),
			(1, 'fr_NC', 'YourCompanyDoesNotUseVAT', 'Votre société/institution est définie comme non assujettie à la TGC (voir Accueil > Configuration > Société/Institution), aussi il n\'y a pas d\'option paramétrage de la TGC.'),
			(1, 'fr_NC', 'SellTaxRate', 'Taux de TGC'),
			(1, 'fr_NC', 'RecuperableOnly', 'Oui pour une TGC \"Perçue Non Récupérable\" dédiée à certains pays comme la France. Gardez la valeur à \"Non\" dans tous les autres cas.'),
			(1, 'fr_NC', 'AmountHTGCTRealReceived', 'HT collectée'),
			(1, 'fr_NC', 'AmountHTGCTRealPaid', 'HT payé'),
			(1, 'fr_NC', 'VATToPay', 'TGC ventes'),
			(1, 'fr_NC', 'VATReceived', 'TGC collectée'),
			(1, 'fr_NC', 'VATToCollect', 'TGC achats'),
			(1, 'fr_NC', 'VATSummary', 'Balance de TGC'),
			(1, 'fr_NC', 'VATPaid', 'TGC payée'),
			(1, 'fr_NC', 'VATCollected', 'TGC récupérée'),
			(1, 'fr_NC', 'PaymentVat', 'Règlement TGC'),
			(1, 'fr_NC', 'VATPayment', 'Règlement TGC'),
			(1, 'fr_NC', 'VATPayments', 'Règlements TGC'),
			(1, 'fr_NC', 'VATRefund', 'Remboursement TGC'),
			(1, 'fr_NC', 'ShowVatPayment', 'Affiche paiement TGC'),
			(1, 'fr_NC', 'CalcModeVATDebt', 'Mode <b>%sTGC sur débit%s</b>.'),
			(1, 'fr_NC', 'CalcModeVATEngagement', 'Mode <b>%sTGC sur encaissement%s</b>.'),
			(1, 'fr_NC', 'RulesResultDue', '- Il comprend les factures impayées, les dépenses, la TGC, les dons qu\'elles soient payées ou non. Il comprend également les salaires versés. <br> - Il est basé sur la date de validation de factures et de la TGC et à la date prévue pour les dépenses. Pour les salaires définis avec le module de salaire, la date de paiement de la valeur est utilisée.'),
			(1, 'fr_NC', 'RulesResultInOut', '- Il comprend les paiements réels effectués sur les factures, les dépenses, la TGC et les salaires. <br> - Il est basé sur les dates de paiement des factures, les dépenses, la TGC et les salaires. La date du don pour le don.'),
			(1, 'fr_NC', 'VATReport', 'Rapport TGC'),
			(1, 'fr_NC', 'VATReportByCustomersInInputOutputMode', 'Rapport par client des TGC collectées et payées'),
			(1, 'fr_NC', 'VATReportByCustomersInDueDebtMode', 'Rapport par client des TGC collectées et payées'),
			(1, 'fr_NC', 'VATReportByQuartersInInputOutputMode', 'Rapport par taux des TGC collectées et payées'),
			(1, 'fr_NC', 'VATReportByQuartersInDueDebtMode', 'Rapport par taux des TGC collectées et payées'),
			(1, 'fr_NC', 'SeeVATReportInInputOutputMode', 'Cliquer sur <b>%sTGC encaissement%s</b> pour mode de calcul standard'),
			(1, 'fr_NC', 'SeeVATReportInDueDebtMode', 'Cliquer sur <b>%sTGC sur débit%s</b> pour mode de calcul avec option sur les débits'),
			(1, 'fr_NC', 'RulesVATInServices', '- Pour les services, le rapport inclut les TGC des règlements effectivement reçus ou émis en se basant sur la date du règlement.'),
			(1, 'fr_NC', 'RulesVATInProducts', '- Pour les biens matériels, il inclut les TGC des factures en se basant sur la date de facture.'),
			(1, 'fr_NC', 'RulesVATDueServices', '- Pour les services, le rapport inclut les TGC des factures dues, payées ou non en se basant sur la date de facture.'),
			(1, 'fr_NC', 'RulesVATDueProducts', '- Pour les biens matériels, il inclut les TGC des factures en se basant sur la date de facture.'),
			(1, 'fr_NC', 'CalculationRuleDesc', 'Pour calculer le total de TGC, il existe 2 modes:<br>Le mode 1 consiste à arrondir la tva de chaque ligne et à sommer cet arrondi.<br>Le mode 2 consiste à sommer la tva de chaque ligne puis à l\'arrondir.<br>Les résultats peuvent différer de quelques centimes. Le mode par défaut est le mode <b>%s</b>.'),
			(1, 'fr_NC', 'ACCOUNTING_VAT_SOLD_ACCOUNT', 'Compte comptable par défaut pour l\'encaissement de TGC - TGC sur les ventes (utilisé si non défini au niveau de la configuration du dictionnaire de TGC)'),
			(1, 'fr_NC', 'ACCOUNTING_VAT_BUY_ACCOUNT', 'Compte comptable par défaut pour le paiement de la TGC - TGC sur les achats (utilisé si non défini au niveau de la configuration du dictionnaire de TGC)'),
			(1, 'fr_NC', 'ACCOUNTING_VAT_PAY_ACCOUNT', 'Compte comptable par défaut pour le paiement de la TGC'),
			(1, 'fr_NC', 'BasedOnTwoFirstLettersOfVATNumberBeingDifferentFromYourCompanyCountry', 'Basé sur les deux premières lettres du numéro de TGC étant différent du code de pays de votre propre entreprise'),
			(1, 'fr_NC', 'BasedOnTwoFirstLettersOfVATNumberBeingTheSameAsYourCompanyCountry', 'Basé sur les deux premières lettres du numéro de TGC étant les mêmes que le code pays de votre propre entreprise'),
			(1, 'fr_NC', 'ImportDataset_tax_vat', 'Paiements de TGC'),
			(1, 'fr_NC', 'LineVATRate', 'Taux de TGC de la ligne'),
			(1, 'fr_NC', 'LineTotalVAT', 'Montant TGC de la ligne'),
			(1, 'fr_NC', 'VATIsUsed', 'Assujetti à la TGC'),
			(1, 'fr_NC', 'VATIsNotUsed', 'Non assujetti à la TGC'),
			(1, 'fr_NC', 'VATIntra', 'Numéro de TGC'),
			(1, 'fr_NC', 'VATIntraShort', 'Num. TGC'),
			(1, 'fr_NC', 'VATIntraCheckDesc', 'Le lien <b>%s</b> permet d\'interroger le service européen de contrôle des numéro de TVA intracommunautaire. Un accès à Internet depuis le serveur est requis pour que ce service fonctionne.'),
			(1, 'fr_NC', 'MainAccountForVatPaymentNotDefined', 'Compte comptable général pour les paiements de TGC sans compte dédié'),
			(1, 'fr_NC', 'AccountancyAreaDescVat', 'Étape %s : Définissez les comptes comptables de chaque taux de TGC utilisé. Pour cela, suivez le menu suivant %s.'),
			(1, 'fr_NC', 'MenuVatAccounts', 'Compte TGC'),
			(1, 'fr_NC', 'VATAccountNotDefined', 'Compte de la TGC non défini'),
			(1, 'fr_NC', 'DefaultBindingDesc', 'Cette page peut être utilisée pour définir un compte par défaut à utiliser pour la ventilation des transactions sur le paiement des salaires, les dons, les charges sociales et fiscales et la TGC lorsqu\'aucun compte spécifique n\'a été défini.'),
			(1, 'fr_NC', 'ProductVatMassChange', 'Changement TGC en masse'),
			(1, 'fr_NC', 'ProductVatMassChangeDesc', 'Cette page permet de modifier les taux de TGC définis sur les produits ou services d\'une valeur vers une autre. Attention, la modification est réalisée en masse sur toute la base de donnée.'),
			(1, 'fr_NC', 'UpdateVAT', 'Modifier TGC'),
			(1, 'fr_NC', 'VATRateForSupplierProduct', 'Taux TGC (pour ce produit/fournisseur)'),
			(1, 'fr_NC', 'VATToUseForSubscriptions', 'Taux de TGC pour les adhésions'),
			(1, 'fr_NC', 'NoVatOnSubscription', 'Pas de TGC sur les adhésions'),
			
			
			
			(1, 'fr_NC', 'VATIsNotUsedForInvoice', 'Franchise en base, TGC non applicable - Article LP.509'),
			(1, 'fr_NC', 'SeparatorThousand', 'Space'),
			(1, 'fr_NC', 'ProfId1NC', 'Identifiant professionnel 1 (RIDET)'),
			(1, 'fr_NC', 'ProfId2NC', 'Identifiant professionnel 2 (RCS)'),
			(1, 'fr_NC', 'LT1', 'TSS'),
			(1, 'fr_NC', 'LT1Paid', 'TSS payée'),
			(1, 'fr_NC', 'LT1Summary', 'Balance de TSS'),
			(1, 'fr_NC', 'LT1Customer', 'RE (TSS) ventes'),
			(1, 'fr_NC', 'newLT1Payment', 'Nouveau paiement de TSS'),
			(1, 'fr_NC', 'LT1Payment', 'Paiement de la TSS'),
			(1, 'fr_NC', 'AmountLT1', 'Montant TSS'),
			(1, 'fr_NC', 'TotalLT1', 'Total TSS');";

    $resql = $db->query($sql);

    if ($resql && mysqli_affected_rows($db->db)) {
        return 1;
    }

    return 0;
}

/**
 * Lightens/darkens a given colour (hex format), returning the altered colour in hex format
 * From: https://gist.github.com/stephenharris/5532899
 *
 * @param str $hex Colour as hexadecimal (with or without hash);
 * @percent float $percent Decimal ( 0.2 = lighten by 20%(), -0.4 = darken by 40%() )
 * @return str Lightened/Darkend colour as hexadecimal (with hash);
 */
function geckanewcaledonia_color_luminance($hex, $percent)
{

    // validate hex string
    $hex = preg_replace('/[^0-9a-f]/i', '', $hex);
    $new_hex = '#';

    if (strlen($hex) < 6) {
        $hex = $hex[0] + $hex[0] + $hex[1] + $hex[1] + $hex[2] + $hex[2];
    }

    // convert to decimal and change luminosity
    for ($i = 0; $i < 3; $i++) {
        $dec = hexdec(substr($hex, $i*2, 2));
        $dec = min(max(0, $dec + $dec * $percent), 255);
        $new_hex .= str_pad(dechex($dec), 2, 0, STR_PAD_LEFT);
    }

    return $new_hex;
}

/**
 * Lightens/darkens a given colour (hex format), returning the altered colour in rgb format
 *
 * @param string|array $rgb Colour as rgb (string, or array);
 * @percent float $percent Decimal ( 0.2 = lighten by 20%(), -0.4 = darken by 40%() )
 * @return array Lightened/Darkend colour as rgb;
 */
function geckanewcaledonia_color_luminance_rgb($rgb, $percent)
{
    if (is_string($rgb)) {
        $rgb = explode(',', $rgb);
    }
    $hex = geckanewcaledonia_rgb2hex($rgb);
    $hex = geckanewcaledonia_color_luminance($hex, $percent);
    return geckanewcaledonia_hex2rgb($hex);
}

function geckanewcaledonia_hex2rgb($hex)
{
    return sscanf($hex, "#%02x%02x%02x");
}
function geckanewcaledonia_rgb2hex($rgb)
{
    return '#' . sprintf('%02x', $rgb[0]) . sprintf('%02x', $rgb[1]) . sprintf('%02x', $rgb[2]);
}
